package com.diagbot.process;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.HighRiskNeoDTO;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleConditionDTO;
import com.diagbot.dto.RuleExtDTO;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.RuleTypeEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.facade.CommonFacade;
import com.diagbot.model.label.PacsLabel;
import com.diagbot.rule.AgeRule;
import com.diagbot.rule.CommonRule;
import com.diagbot.rule.LisRule;
import com.diagbot.rule.VitalRule;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.MsgUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.Drug;
import com.diagbot.vo.RuleVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description: 高风险总入口
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class HighRiskProcess {

    @Autowired
    CommonRule commonRule;
    @Autowired
    CommonFacade commonFacade;
    @Autowired
    AgeRule ageRule;
    @Autowired
    LisRule lisRule;
    @Autowired
    VitalRule vitalRule;

    /**
     * 处理业务——高危药品
     *
     * @param wordCrfDTO
     * @param res
     */
    public void processDrug(WordCrfDTO wordCrfDTO, IndicationDTO res) {
        List<BillMsg> highRiskList = res.getHighRiskList();
        List<Drug> drugList = wordCrfDTO.getDrugOrder();
        for (Drug drug : drugList) {
            RuleVO ruleVO = getRuleVO(drug);
            // 1、获取高危相关数据
            if (ruleVO != null) {
                RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
                if (ruleExtDTO == null) {
                    continue;
                }
                // 2、简化拼接对象，用于开单项提示语
                RuleSimpleDTO ruleSimpleDTO = new RuleSimpleDTO();
                BeanUtil.copyProperties(ruleExtDTO, ruleSimpleDTO);
                // 3、规则匹配
                List<RuleConditionDTO> ruleConditionDTOList = ruleExtDTO.getRuleConditionDTOList();
                for (RuleConditionDTO ruleConditionDTO : ruleConditionDTOList) {
                    List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                    ruleSimpleDTO.setMsg(ruleConditionDTO.getMsg());
                    for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList ) {
                        if (ruleBaseDTO.getBaseLibName().equals(ruleVO.getForm())) {
                            commonRule.highRisk(ruleSimpleDTO, highRiskList);
                        }
                    }
                }
            }
        }
    }

    /**
     * 处理业务——高危手术
     *
     * @param wordCrfDTO
     * @param res
     */
    public void processOperation(WordCrfDTO wordCrfDTO, IndicationDTO res) {
        List<BillMsg> highRiskList = res.getHighRiskList();
        List<Item> operationList = wordCrfDTO.getOperationOrder();
        for (Item operate : operationList) {
            RuleVO ruleVO = getRuleVO(operate);
            // 1、获取高危相关数据
            if (ruleVO != null) {
                RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
                if (ruleExtDTO == null) {
                    continue;
                }
                // 2、简化拼接对象，用于开单项提示语
                RuleSimpleDTO ruleSimpleDTO = new RuleSimpleDTO();
                BeanUtil.copyProperties(ruleExtDTO, ruleSimpleDTO);
                // 3、规则匹配
                List<RuleConditionDTO> ruleConditionDTOList = ruleExtDTO.getRuleConditionDTOList();
                for (RuleConditionDTO ruleConditionDTO : ruleConditionDTOList) {
                    if (ruleConditionDTO.getHasSubCond().equals(0)) { // 3,4级高危手术，本身高危
                        BillMsg highRisk = MsgUtil.getCommonHighRiskMsg("", ruleExtDTO.getInputName(), TypeEnum.operation.getName());
                        highRiskList.add(highRisk);
                    } else { // 2级高危手术
                        List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                        for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList) {
                            if (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType()) != null) { // 避免空指针
                                switch (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType())) {
                                    case Age: // 年龄
                                        ageRule.highOperation(wordCrfDTO, ruleBaseDTO, highRiskList, ruleSimpleDTO);
                                        break;
                                    case Disease: // 诊断
                                        commonRule.compareItemWithHighRisk(wordCrfDTO.getDiag(), ruleBaseDTO, highRiskList, ruleSimpleDTO);
                                        break;
                                    case LisSubName: // 化验
                                        lisRule.highOperation(wordCrfDTO.getLis(), ruleBaseDTO, highRiskList, ruleSimpleDTO);
                                        break;
                                    case VitalResult: //体格检查结果
                                    case Vital: // 体格检查项目
                                        vitalRule.highOperation(wordCrfDTO.getVitalLabel(), ruleBaseDTO, highRiskList, ruleSimpleDTO);
                                        break;
                                    case PacsResult: // 禁用辅助检查描述
                                        // 【辅检结果数据来源】
                                        PacsLabel pacsLabel = wordCrfDTO.getPacsLabel();
                                        List<Item> pacsDescList = pacsLabel.getRes();
                                        commonRule.compareItemWithHighRisk(pacsDescList, ruleBaseDTO, highRiskList, ruleSimpleDTO);
                                        break;
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * 获取高危入参——药品
     *
     * @param bean
     * @return
     */
    public RuleVO getRuleVO(Drug bean) {
        if (StringUtil.isNotBlank(bean.getUniqueName())) {
            RuleVO ruleVO = new RuleVO();
            ruleVO.setInputName(bean.getName());
            ruleVO.setLibName(bean.getUniqueName());
            ruleVO.setLibType(LexiconEnum.Medicine.getKey());
            ruleVO.setLibTypeName(TypeEnum.drug.getName());
            ruleVO.setDateValue(bean.getDateValue());
            ruleVO.setSuffix(String.valueOf(RuleTypeEnum.high.getKey()));
            ruleVO.setForm(bean.getForm());
            return ruleVO;
        }
        return null;
    }

    /**
     * 获取高危入参——手术
     *
     * @param bean
     * @return
     */
    public RuleVO getRuleVO(Item bean) {
        if (StringUtil.isNotBlank(bean.getUniqueName())) {
            RuleVO ruleVO = new RuleVO();
            ruleVO.setInputName(bean.getName());
            ruleVO.setLibName(bean.getUniqueName());
            ruleVO.setLibType(LexiconEnum.Operation.getKey());
            ruleVO.setLibTypeName(TypeEnum.operation.getName());
            ruleVO.setDateValue(bean.getDateValue());
            ruleVO.setSuffix(String.valueOf(RuleTypeEnum.high.getKey()));
            return ruleVO;
        }
        return null;
    }

    /**
     * 处理业务
     *
     * @param highRiskNeoDTOList
     * @param res
     */
    public void process(List<HighRiskNeoDTO> highRiskNeoDTOList, IndicationDTO res, WordCrfDTO wordCrfDTO) {
        List<BillMsg> highRiskList = new ArrayList<>();
        CommonRule commonRule = new CommonRule();
        for (HighRiskNeoDTO highRiskNeoDTO : highRiskNeoDTOList) {
            System.out.println(highRiskNeoDTO);
            String type = highRiskNeoDTO.getType();
            if (highRiskNeoDTO.getIsHighrisk() == true) {
                commonRule.highRisk(highRiskNeoDTO, highRiskList);
            } else {
                // 复杂情况高危手术
                if (TypeEnum.operation.getName().equals(type)) {
                    List<NodeNeoDTO> factor = highRiskNeoDTO.getFactor();
                    for (NodeNeoDTO nodeNeoDTO : factor) {
                        switch (TypeEnum.getEnum(nodeNeoDTO.getTermtype())) {
                            case disease:
                            case lis:
                            case pacs:
                            case vital:
                            case age:
                                commonRule.highRiskComplex(nodeNeoDTO, highRiskList, highRiskNeoDTO);
                                break;
                        }
                    }
                }
            }
        }
        res.setHighRiskList(highRiskList);
    }
}
